/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import java.math.BigInteger;

import ru.ispras.fortress.data.Data;
import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.data.DataTypeId;
import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.util.InvariantChecks;

public final class DataGeneratorVariable {
  private final String name;
  private final Node node;
  private NodeValue value;

  public DataGeneratorVariable(final String name, final Node node) {
    InvariantChecks.checkNotNull(name);
    InvariantChecks.checkNotNull(node);

    checkNodeKind(name, node);
    checkDataType(name, node);

    this.name = name;
    this.node = node;
    this.value = null;
  }

  private static void checkNodeKind(final String name, final Node node) {
    if (node.getKind() != Node.Kind.VALUE && node.getKind() != Node.Kind.VARIABLE) {
      throw new IllegalArgumentException(String.format(
          "%s has unsupported node kind %s. Variable or value is expected.", name, node.getKind()));
    }
  }

  private static void checkDataType(final String name, final Node node) {
    if (!node.isType(DataTypeId.BIT_VECTOR)) {
      throw new IllegalArgumentException(String.format(
          "%s has unsupported data type %s. Bit vector is expected.", name, node.getDataType()));
    }
  }

  private void checkDataType(final String name, final DataType type) {
    if (!getType().equals(type)) {
      throw new IllegalArgumentException(String.format(
          "%s is illegal for %s. %s is expected.", type, name, getType()));
    }
  }

  private void checkNotValueSet() {
    if (isValueSet()) {
      throw new IllegalStateException(String.format(
          "%s is already assigned a value.", name));
    }
  }

  public String getName() {
    return name;
  }

  public DataType getType() {
    return node.getDataType();
  }

  public int getSize() {
    return getType().getSize();
  }

  public boolean isUnknown() {
    return Utils.isUnknownVariable(node);
  }

  public boolean isValueSet() {
    return isUnknown() ? null != value : true;
  }

  public NodeValue getValue() {
    final NodeValue result = isUnknown() ? value : (NodeValue) node;
    if (null == result) {
      throw new IllegalStateException(name + " is not assigned a value.");
    }
    return result;
  }

  public BigInteger integerValue() {
    return bitVectorValue().bigIntegerValue();
  }

  public BitVector bitVectorValue() {
    return getValue().getBitVector();
  }

  public void setValue(final int value) {
    setValue(Data.newBitVector(value, getSize()));
  }

  public void setValue(final long value) {
    setValue(Data.newBitVector(value, getSize()));
  }

  public void setValue(final BigInteger value) {
    setValue(Data.newBitVector(value, getSize()));
  }

  public void setValue(final BitVector value) {
    setValue(NodeValue.newBitVector(value));
  }

  public void setValue(final Data value) {
    setValue(new NodeValue(value));
  }

  public void setValue(final NodeValue value) {
    InvariantChecks.checkNotNull(value);
    checkNotValueSet();

    checkDataType(name, value.getDataType());
    this.value = value;
  }
}
