/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.stub;

import java.math.BigInteger;
import java.util.LinkedHashMap;
import java.util.Map;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.data.DataTypeId;
import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.randomizer.Randomizer;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.generator.Utils;

final class TdpRandom extends TdpBase {
  public static final String NAME = "random";

  public TdpRandom() {
    super(NAME);
  }

  @Override
  protected TestData generateData(final TestBaseQuery query) {
    final BigInteger min = Utils.hasParameter(query, "min") ?
        Utils.getParameterAsBigInteger(query, "min") : null;

    final BigInteger max = Utils.hasParameter(query, "max") ?
        Utils.getParameterAsBigInteger(query, "max") : null;

    final int size = Utils.hasParameter(query, "size") ?
        Utils.getParameterAsInt(query, "size") : -1;

    final boolean signExtend = isSignExtend(query);

    final Map<String, Node> unknowns = Utils.extractUnknown(query);
    final Map<String, Node> outputData = new LinkedHashMap<>();

    for (final Map.Entry<String, Node> e : unknowns.entrySet()) {
      final String name = e.getKey();
      final DataType type = e.getValue().getDataType();

      final Node value;
      if (DataTypeId.BIT_VECTOR == type.getTypeId()) {
        final int bitSize = size != -1 ? size : type.getSize();
        final BitVector data = newRandomData(bitSize, min, max);

        value = NodeValue.newBitVector(type.getSize() == bitSize ?
            data : data.resize(type.getSize(), signExtend));
      } else {
        throw new IllegalArgumentException(String.format(
            "The %s variable has unupported type: %s", name, type));
      }

      outputData.put(name, value);
    }

    return new TestData(outputData);
  }

  private static BitVector newRandomData(
      final int bitSize,
      final BigInteger min,
      final BigInteger max) {
    final BitVector result;

    if (min != null && max != null) {
      final BigInteger bi = Randomizer.get().nextBigIntegerRange(min, max);
      result = BitVector.valueOf(bi, bitSize);
    } else {
      result = BitVector.newEmpty(bitSize);
      Randomizer.get().fill(result);
    }

    return result;
  }

  private static boolean isSignExtend(final TestBaseQuery query) {
    final Object parameter = Utils.getParameter(query, "sign_extend");
    return parameter instanceof Boolean ? (Boolean) parameter : false; 
  }
}
